//    ******************************************
//    *                                        *
//    *  Sensor:   MuenzPruefer                *
//    *                                        *
//    *  methods:     (boolean) getPulse()     *
//    *                                        *
//    *  author:   Jochen Viehoff              *
//    *                                        *
//    ******************************************


import java.io.*;
import java.util.*;
import javax.comm.*;                          // commAPI for serial communication

public class MuenzPruefer implements SerialPortEventListener {
    
    static Enumeration        portList;       // list for connected ports
    static CommPortIdentifier portId;
    
    static SerialPort   serialPort;
    static OutputStream outputStream;
    static InputStream  inputStream;
    
    static boolean pulse;                     // boolean value: coin count
    static int     data;
    
    //    ******************************************
    //      constructor
    //    ******************************************
    
    public MuenzPruefer() {
        
        portList = CommPortIdentifier.getPortIdentifiers();
        
        // locking for serial ports
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            System.out.println(portId.getName());
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                System.out.println("Found: SerialPort");
                if (portId.getName().equals("COM2")) {
                    System.out.println("Found COM2");
                    
                    // open port COM1
                    try {
                        serialPort = (SerialPort) portId.open("MuenzPruefer", 2000);
                        System.out.println("open port: done");
                    } catch (PortInUseException e) {System.out.println(e);}
                    
                    // get the streams from/to port COM2
                    try {
                        inputStream = serialPort.getInputStream();
                        outputStream= serialPort.getOutputStream();
                    } catch (IOException e) {System.out.println(e);}
                    
                    try {
                        serialPort.addEventListener(this);
                    } catch (TooManyListenersException e) {System.out.println(e);}
                    
                    // start listening to the port
                    serialPort.notifyOnDataAvailable(true);
                    
                    // set port parameters
                    try {
                        serialPort.setSerialPortParams(9600,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                    } catch (UnsupportedCommOperationException e) {System.out.println(e);}
                }
            }
        }
    }
    
    //    ******************************************
    //      public methods
    //    ******************************************
    
    public static boolean getPulse() {
        
        boolean p;
        p = pulse;
        pulse = false;
        return p;              // return pulse
    }
    
    //    ******************************************
    //      SerialEvent handling
    //    ******************************************
    
    public void serialEvent(SerialPortEvent e) {
        
        // Create a StringBuffer and int to receive input data.
        StringBuffer inputBuffer = new StringBuffer();
        int newData=0;
        int i=0;
        String s1="";
        
        // Determine type of event
        switch (e.getEventType()) {
            
            // Read data until -1 is returned
            case SerialPortEvent.DATA_AVAILABLE:
                while (newData != -1) {
                    try {
                        newData = inputStream.read();
                        
                        if (newData == -1) {
                            System.out.println("-1 received !!");
                            break;
                        }
                        if (newData == 10) {
                            try{data=Integer.parseInt(s1.trim());}
                            catch (NumberFormatException fne){}
                            
                            pulse = true;
                            
                            s1="";
                        }
                        else {
                            s1=s1+(char)newData;
                        }
                    }
                    catch (IOException ex) {
                        System.err.println(ex);
                        return;
                    }
                }
                
                break;
                
                // If break event append BREAK RECEIVED message.
                
            case SerialPortEvent.BI:
                System.out.println("\n--- BREAK RECEIVED ---\n");
        }
        
    }
}
