// **********************************************
// *                                            *
// *   Java Applikation:  Video                 *
// *                                            *
// *       Tina's FrameDrucker:                 *
// *       print & remove single Frame          *
// *       from Quicktime movie                 *
// *       (full screen mode)                   *
// *                                            *
// *   author:  jochen viehoff                  *
// *                                            *
// *                      interface.khm.de      *
// *                        codekit.khm.de      *
// *                                            *
// **********************************************

import java.awt.*;
import java.awt.image.*;
import java.io.*;

import quicktime.std.StdQTConstants;
import quicktime.*;
import quicktime.qd.*;
import quicktime.io.*;
import quicktime.std.movies.*;

import quicktime.app.players.*;
import quicktime.app.QTFactory;
import quicktime.app.time.*;
import quicktime.app.display.*;

public class Video extends Frame implements Runnable,StdQTConstants, QDConstants {
    
    Movie          mv;
    QTDrawable     myPlayer;
    Thread         runThread;
    Image          img;
    BufferedImage  bi;
    
    int   dpi    = 130;                 // print resolution
    int   tFrame = 100;                 // time of one frame
    
    int width    = 720;                 // display size
    int height   = 576;
    int delay    = 200;                 // delay time
    
    int dim      = width*height;
    int [] pixel = new int [dim];
    
    Robot pixelRobot;
    Rectangle screenCapture = new Rectangle(151,96,width,height);   // capture area on full screen
    
    MuenzPruefer myPruefer;
    
    // *************************************************
    //  Video: main method
    // *************************************************
    public static void main(String args[]) {
        
        try {
            QTSession.open();
            Video tinasVideo = new Video();
            
        } catch (Exception e) {
            e.printStackTrace();
            QTSession.close();
        }
    }
    
    // *************************************************
    //  Constructor: Objekt Video
    // *************************************************
    Video() throws Exception {
        
        QTCanvas myQTCanvas = new QTCanvas(QTCanvas.kInitialSize, 0.5f, 0.5f);
        FullScreenWindow full = new FullScreenWindow(new FullScreen(),this);
        full.setBackground(Color.black);
        full.add(myQTCanvas);
        full.show();
        
        //  Quicktime: get Movie
        QTFile qtf = new QTFile(QTFactory.findAbsolutePath("dvpal.mov"));        
//        QTFile qtf = new QTFile(QTFactory.findAbsolutePath("lift2.mov"));
        
        OpenMovieFile movieFile = OpenMovieFile.asRead(qtf);
        mv = Movie.fromFile(movieFile);
        
        //  Quicktime: create new MoviePresenter
        myPlayer = new MoviePresenter(mv);
        
        // link player to canvas
        myQTCanvas.setClient(myPlayer, true);
        pack();
        
        // start pixel robot for screen capture
        try { pixelRobot = new Robot(); }
        catch(AWTException awte) {System.out.println(awte);}
        
        // start Muenzpruefer
        MuenzPruefer myPruefer = new MuenzPruefer();
        
        // start: control thread
        if (runThread==null) {
            Thread runThread = new Thread(this);
            runThread.start();
        }
        
        // start: movie playback
        mv.setRate(1);
    }
    
    // *************************************************
    //  Thread run method
    // *************************************************
    public void run() {
        
        boolean running  = true;
        boolean printing = true;
        
        while (running) {
            
            try {Thread.sleep(delay);}
            catch (InterruptedException e){}
            
            if (myPruefer.getPulse()) printFrame();
            
 //          if (printing) printFrame();
 //           printing = false;
            
            try{
                if (mv.isDone()) {
                    mv.setTimeValue(0);       // restart movie (looping)
                }
            }
            catch (Exception qte){}
        }
    }
    
    // *************************************************
    //  Print one Frame
    // *************************************************
    public void printFrame()  {
        
        try {
            mv.setRate(0);                                             // stop movie
            
            bi   = pixelRobot.createScreenCapture(screenCapture);      // grab pixels from screen & create image
            pixel = bi.getRGB(0,0,width,height,pixel,0,width);
            img = createImage(new MemoryImageSource(width,height,pixel,0,width));
            
            PrintResolution pr = new PrintResolution(img,dpi,false);   // print image via PrintResolution
            pr.print();
            pr = null;
            
            mv.deleteSegment(mv.getTime(),tFrame);     // cut frame
            saveMovie();                               // save movie
            mv.setRate(1);                             // continue movie
        }
        catch (Exception qte){System.out.println(qte);}
        
    }
    public void saveMovie() {
        
        try {
            
            QTFile qtf = new QTFile("C:\\Tina\\VideoPlayer\\currentMovie.mov");
            
            mv.flatten(
            0,
            qtf,
            StdQTConstants.kMoviePlayer,
            IOConstants.smSystemScript,
            StdQTConstants.createMovieFileDeleteCurFile,
            StdQTConstants.movieInDataForkResID,
            qtf.getName());
        } catch (QTException qte) {}
    }
}

